/****************************************************************************
* libogc - libjpeg test
****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <gccore.h>		/*** Wrapper to include common libogc headers ***/
#include <ogcsys.h>		/*** Needed for console support ***/
#include <string.h>
#include <jpeg/jpgogc.h>

/*** External Picture ***/
extern char     picdata[];
extern int      piclength;

/*** 2D Video Globals ***/
GXRModeObj     *vmode;		/*** Graphics Mode Object ***/
u32            *xfb[2] = { NULL, NULL };
				/*** Framebuffers ***/
int             whichfb = 0;	/*** Frame buffer toggle ***/

/****************************************************************************
* Initialise Video
*
* Before doing anything in libogc, it's recommended to configure a video
* output.
****************************************************************************/
static void
Initialise(void)
{
    VIDEO_Init();		/*** ALWAYS CALL FIRST IN ANY LIBOGC PROJECT!
				     Not only does it initialise the video 
				     subsystem, but also sets up the ogc os
				***/

    PAD_Init();			/*** Initialise pads for input ***/

	/*** Try to match the current video display mode
	     using the higher resolution interlaced.
	
	     So NTSC/MPAL gives a display area of 640x480
	     PAL display area is 640x528
	***/
    switch (VIDEO_GetCurrentTvMode()) {
    case VI_NTSC:
	vmode = &TVNtsc480IntDf;
	break;

    case VI_PAL:
	vmode = &TVPal528IntDf;
	break;

    case VI_MPAL:
	vmode = &TVMpal480IntDf;
	break;

    default:
	vmode = &TVNtsc480IntDf;
	break;
    }

	/*** Let libogc configure the mode ***/
    VIDEO_Configure(vmode);

	/*** Now configure the framebuffer. 
	     Really a framebuffer is just a chunk of memory
	     to hold the display line by line.
	***/

    xfb[0] = (u32 *) MEM_K0_TO_K1(SYS_AllocateFramebuffer(vmode));
	/*** I prefer also to have a second buffer for double-buffering.
	     This is not needed for the console demo.
	***/
    xfb[1] = (u32 *) MEM_K0_TO_K1(SYS_AllocateFramebuffer(vmode));

	/*** Define a console ***/
    console_init(xfb[0], 20, 64, vmode->fbWidth, vmode->xfbHeight,
		 vmode->fbWidth * 2);

	/*** Clear framebuffer to black ***/
    VIDEO_ClearFrameBuffer(vmode, xfb[0], COLOR_BLACK);
    VIDEO_ClearFrameBuffer(vmode, xfb[1], COLOR_BLACK);

	/*** Set the framebuffer to be displayed at next VBlank ***/
    VIDEO_SetNextFramebuffer(xfb[0]);

	/*** Get the PAD status updated by libogc ***/
    VIDEO_SetPostRetraceCallback(PAD_ScanPads);
    VIDEO_SetBlack(0);

	/*** Update the video for next vblank ***/
    VIDEO_Flush();

    VIDEO_WaitVSync();		/*** Wait for VBL ***/
    if (vmode->viTVMode & VI_NON_INTERLACE)
	VIDEO_WaitVSync();

}

/****************************************************************************
* Main
*
* jpeg.c - Demo for libjpeg wrapper function
****************************************************************************/
int
main()
{

    JPEGIMG         jpeg;
    int             row,
                    col,
                    pix,
                    offset;
    unsigned int   *jpegout;

    memset(&jpeg, 0, sizeof(JPEGIMG));

    Initialise();	/*** Setup video ***/

    printf("libjpeg 6b demo\n");

  /*** Set the Mandatory values ***/
    jpeg.inbuffer = picdata;
    jpeg.inbufferlength = piclength;

  /*** Call decompressor ***/
    JPEG_Decompress(&jpeg);

    printf("Press A to show image\n");

    while (!(PAD_ButtonsDown(0) & PAD_BUTTON_A));
    while (PAD_ButtonsDown(0) & PAD_BUTTON_A);

    whichfb ^= 1;
    pix = 0;

    jpegout = (unsigned int *) jpeg.outbuffer;

    offset = 0;
    for (row = 0; row < jpeg.height; row++) {
	for (col = 0; col < (jpeg.width >> 1); col++)
	    xfb[whichfb][offset + col] = jpegout[pix++];

	offset += 320;
    }

  /*** IMPORTANT - RELEASE THE MEMORY BUFFER ***/
    free(jpeg.outbuffer);

  /*** And show the image ***/
    VIDEO_SetNextFramebuffer(xfb[whichfb]);
    VIDEO_Flush();
    VIDEO_WaitVSync();

    while (1);

    return 0;		/*** Keep gcc happy ***/

}
